<?php
/**
 * Ce fichier contient l'API de gestion des types de contrôle.
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Charge ou recharge la configuration des dashboards à partir de leur fichier YAML.
 * La fonction compile les dashboards dans un cache unique sécurisé.
 *
 * @api
 *
 * @return bool `false` si une erreur s'est produite, `true` sinon.
 */
function dashboard_charger() {

	// Retour de la fonction
	$retour = true;

	// On recherche les dashboards directement par leur fichier YAML de configuration car il est
	// obligatoire. La recherche s'effectue dans le path en utilisant le dossier relatif fourni.
	if ($fichiers = find_all_in_path('ezcheck/dashboards/', '.+[.](yaml|json)$')) {
		// Initialisation du tableau des dashboard
		$dashboards = array();

		foreach ($fichiers as $_config => $_chemin) {
			// Détermination de l'extension du fichier json ou yaml.
			$extension = pathinfo($_config, PATHINFO_EXTENSION);

			// L'identifiant du dashboard est son nom de fichier sans extension
			$dashboard_id = basename($_config, ".$extension");

			// Initialisation de la description par défaut du type de contrôle
			$description_defaut = array(
				'identifiant'   => $dashboard_id,
				'nom'           => $dashboard_id,
				'description'   => '',
				'icone'         => 'dashboard_defaut-24.png',
				'groupes'       => array(),
			);

			// Lecture et décodage du fichier YAML ou JSON en structure de données PHP.
			if ($extension === 'json') {
				include_spip('inc/flock');
				lire_fichier($_chemin, $dashboard_contenu);
				$description = json_decode($dashboard_contenu, true);
			} else {
				include_spip('inc/yaml');
				$description = yaml_decode_file($_chemin, array('include' => false));
			}

			// Complétude de la description avec les valeurs par défaut
			$description = array_merge($description_defaut, $description);

			// On reformate le tableau des groupes pour que l'index soit l'identifiant
			$groupes = array();
			foreach ($description['groupes'] as $_groupe) {
				$groupes[$_groupe['identifiant']] = $_groupe;
			}
			$description['groupes'] = $groupes;

			// Permettre à des plugins de compléter la description du dashboard, en particulier, d'ajouter
			// des groupes et/ou des types de contrôle à un groupe.
			$flux = array(
				'args' => array(
					'dashboard'  => $dashboard_id,
				),
				'data' => $description
			);
			$description = pipeline('ezdashboard_declarer', $flux);

			// On ajoute le dashboard nouveau ou modifié
			$dashboards[$dashboard_id] = $description;
		}

		// Etant donné que le nombre dashboard est réduit tout comme les informations qui le compose on choisit
		// de l'écrire systématiquement.
		// -- Initialisation de l'identifiant du cache des dashboards
		$cache = array(
			'nom' => 'dashboards',
		);
		// -- Ecriture du cache
		include_spip('inc/ezcache_cache');
		cache_ecrire('ezcheck', 'dashboard', $cache, $dashboards);
	}

	return $retour;
}

/**
 * Retourne la description complète de la page, de la composition explicite ou de la composition virtuelle demandée ou
 * une information donnée uniquement.
 * La description est stockée en base de données, certains champs sont recalculés avant d'être fournis.
 *
 * @api
 *
 * @param string $dashboard_id Identifiant de la page ou de la composition.
 * @param bool   $traiter_typo Indique si les données textuelles doivent être retournées brutes ou si elles doivent
 *                             être traitées en utilisant la fonction typo.
 *
 * @return array La description complète. Les champs de type tableau sont systématiquement désérialisés et si demandé,
 *               les champs textuels peuvent être traités avec la fonction typo().
 */
function dashboard_lire($dashboard_id, $traiter_typo = false) {

	// On met en cache mémoire statique les dashboards déjà lus.
	static $dashboards_charges = array();

	// On vérifie si le dashboard demandé n'est pas déjà stocké : si oui, la description est renvoyée immédiatement
	if (isset($dashboards_charges[$traiter_typo][$dashboard_id])) {
		$dashboard = $dashboards_charges[$traiter_typo][$dashboard_id];
	} else {
		// Il faut charger le dashboard depuis le cache. On initialise avec le tableau qui est indique une erreur.
		$dashboard = array();

		// Chargement de tous les dashboards connus à partir du fichier cache.
		$dashboards_caches = dashboard_repertorier();

		// Sauvegarde de la description de la page pour une consultation ultérieure dans le même hit.
		if (isset($dashboards_caches[$dashboard_id])) {
			// Extraction du dashboard demandé
			$dashboard = $dashboards_caches[$dashboard_id];

			// Traitements des champs textuels
			if ($traiter_typo) {
				$dashboard['nom'] = typo($dashboard['nom']);
				if ($dashboard['description']) {
					$dashboard['description'] = typo($dashboard['description']);
				}
				foreach ($dashboard['groupes'] as $_id => $_groupe) {
					$dashboard['groupes'][$_id]['nom'] = typo($dashboard['groupes'][$_id]['nom']);
				}
			}

			// Stockage de la description du dashboard lu
			$dashboards_charges[$traiter_typo][$dashboard_id] = $dashboard;
		} else {
			// En cas d'erreur stocker un dashboard vide
			$dashboards_charges[$traiter_typo][$dashboard_id] = array();
		}
	}

	return $dashboard;
}

/**
 * Renvoie l'information brute demandée pour l'ensemble des contrôles utilisés
 * ou toute les descriptions si aucune information n'est explicitement demandée.
 *
 * @param string $information Identifiant d'un champ de la description d'un contrôle.
 *                            Si l'argument est vide, la fonction renvoie les descriptions complètes et si l'argument est
 *                            un champ invalide la fonction renvoie un tableau vide.
 *
 * @return array Tableau de la forme `[type_controle] = information ou description complète`. Les champs textuels
 *               sont retournés en l'état, le timestamp `maj n'est pas fourni.
 */
function dashboard_repertorier($information = '') {

	// Initialiser le tableau de sortie en cas d'erreur
	$dashboards = $information ? '' : array();

	// Les dashboards sont stockées dans un cache sécurisé géré par Cache Factory.
	// -- Initialisation de l'identifiant du cache des dashboards
	$cache = array(
		'nom' => 'dashboards',
	);

	include_spip('inc/ezcache_cache');
	if ($descriptions = cache_lire('ezcheck', 'dashboard', $cache)) {
		if ($information) {
			// Si $information n'est pas une colonne valide array_column retournera un tableau vide.
			if ($informations = array_column($descriptions, $information, 'identifiant')) {
				$dashboards = $informations;
			}
		} else {
			$dashboards = $descriptions;
		}
	}

	return $dashboards;
}
